@@ -11,7 +11,7 @@
 struct file_ll {
     char *dname;
     int32_t stored_count;
-    int32_t email_count;
+    int32_t item_count;
     int32_t skip_count;
     int32_t type;
 };
@@ -28,12 +28,12 @@ pst_file pstfile;
 
 void create_enter_dir(struct file_ll* f, pst_item *item)
 {
-    f->email_count  = 0;
+    pst_convert_utf8(item, &item->file_as);
+    f->item_count   = 0;
     f->skip_count   = 0;
     f->type         = item->type;
-    f->stored_count = (item->folder) ? item->folder->email_count : 0;
-    f->dname        = (char*) xmalloc(strlen(item->file_as)+1);
-    strcpy(f->dname, item->file_as);
+    f->stored_count = (item->folder) ? item->folder->item_count : 0;
+    f->dname        = strdup(item->file_as.str);
 }
 
 
@@ -43,97 +43,105 @@ void close_enter_dir(struct file_ll *f)
 }
 
 
-void process(pst_item *outeritem, pst_desc_ll *d_ptr)
+void process(pst_item *outeritem, pst_desc_tree *d_ptr)
 {
     struct file_ll ff;
     pst_item *item = NULL;
+    char *result = NULL;
+    size_t resultlen = 0;
 
     DEBUG_ENT("process");
     memset(&ff, 0, sizeof(ff));
     create_enter_dir(&ff, outeritem);
 
     while (d_ptr) {
-        DEBUG_MAIN(("main: New item record, d_ptr = %p.\n", d_ptr));
         if (!d_ptr->desc) {
-            DEBUG_WARN(("main: ERROR ?? item's desc record is NULL\n"));
+            DEBUG_WARN(("ERROR item's desc record is NULL\n"));
             ff.skip_count++;
         }
         else {
-            DEBUG_MAIN(("main: Desc Email ID %x [d_ptr->id = %x]\n", d_ptr->desc->id, d_ptr->id));
+            DEBUG_INFO(("Desc Email ID %"PRIx64" [d_ptr->d_id = %"PRIx64"]\n", d_ptr->desc->i_id, d_ptr->d_id));
 
-            item = pst_parse_item(&pstfile, d_ptr);
-            DEBUG_MAIN(("main: About to process item @ %p.\n", item));
+            item = pst_parse_item(&pstfile, d_ptr, NULL);
+            DEBUG_INFO(("About to process item @ %p.\n", item));
             if (item) {
                 if (item->message_store) {
                     // there should only be one message_store, and we have already done it
-                    DIE(("main: A second message_store has been found. Sorry, this must be an error.\n"));
+                    DIE(("A second message_store has been found. Sorry, this must be an error.\n"));
                 }
 
                 if (item->folder && d_ptr->child) {
                     // if this is a folder, we want to recurse into it
-                    printf("Folder \"%s\"\n", item->file_as);
+                    pst_convert_utf8(item, &item->file_as);
+                    printf("Folder \"%s\"\n", item->file_as.str);
                     process(item, d_ptr->child);
 
                 } else if (item->contact && (item->type == PST_TYPE_CONTACT)) {
+                    if (!ff.type) ff.type = item->type;
                     // Process Contact item
                     if (ff.type != PST_TYPE_CONTACT) {
-                        DEBUG_MAIN(("main: I have a contact, but the folder isn't a contacts folder. Processing anyway\n"));
+                        DEBUG_INFO(("I have a contact, but the folder isn't a contacts folder. Processing anyway\n"));
                     }
                     printf("Contact");
-                    if (item->contact->fullname)
-                        printf("\t%s", pst_rfc2426_escape(item->contact->fullname));
+                    if (item->contact->fullname.str)
+                        printf("\t%s", pst_rfc2426_escape(item->contact->fullname.str, &result, &resultlen));
                     printf("\n");
 
-                } else if (item->email && (item->type == PST_TYPE_NOTE || item->type == PST_TYPE_REPORT || item->type == PST_TYPE_OTHER)) {
+                } else if (item->email && ((item->type == PST_TYPE_NOTE) || (item->type == PST_TYPE_SCHEDULE) || (item->type == PST_TYPE_REPORT))) {
+                    if (!ff.type) ff.type = item->type;
                     // Process Email item
-                    if ((ff.type != PST_TYPE_NOTE) && (ff.type != PST_TYPE_REPORT) && (ff.type != PST_TYPE_OTHER)) {
-                        DEBUG_MAIN(("main: I have an email, but the folder isn't an email folder. Processing anyway\n"));
+                    if ((ff.type != PST_TYPE_NOTE) && (ff.type != PST_TYPE_SCHEDULE) && (ff.type != PST_TYPE_REPORT)) {
+                        DEBUG_INFO(("I have an email, but the folder isn't an email folder. Processing anyway\n"));
                     }
                     printf("Email");
-                    if (item->email->outlook_sender_name)
-                        printf("\tFrom: %s", item->email->outlook_sender_name);
-                    if (item->email->subject && item->email->subject->subj)
-                        printf("\tSubject: %s", item->email->subject->subj);
+                    if (item->email->outlook_sender_name.str)
+                        printf("\tFrom: %s", item->email->outlook_sender_name.str);
+                    if (item->subject.str)
+                        printf("\tSubject: %s", item->subject.str);
                     printf("\n");
 
                 } else if (item->journal && (item->type == PST_TYPE_JOURNAL)) {
+                    if (!ff.type) ff.type = item->type;
                     // Process Journal item
                     if (ff.type != PST_TYPE_JOURNAL) {
-                        DEBUG_MAIN(("main: I have a journal entry, but folder isn't specified as a journal type. Processing...\n"));
+                        DEBUG_INFO(("I have a journal entry, but folder isn't specified as a journal type. Processing...\n"));
                     }
-                    if (item->email && item->email->subject && item->email->subject->subj)
-                        printf("Journal\t%s\n", pst_rfc2426_escape(item->email->subject->subj));
+                    if (item->subject.str)
+                        printf("Journal\t%s\n", pst_rfc2426_escape(item->subject.str, &result, &resultlen));
 
                 } else if (item->appointment && (item->type == PST_TYPE_APPOINTMENT)) {
+                    char time_buffer[30];
+                    if (!ff.type) ff.type = item->type;
                     // Process Calendar Appointment item
-                    DEBUG_MAIN(("main: Processing Appointment Entry\n"));
+                    DEBUG_INFO(("Processing Appointment Entry\n"));
                     if (ff.type != PST_TYPE_APPOINTMENT) {
-                        DEBUG_MAIN(("main: I have an appointment, but folder isn't specified as an appointment type. Processing...\n"));
+                        DEBUG_INFO(("I have an appointment, but folder isn't specified as an appointment type. Processing...\n"));
                     }
                     printf("Appointment");
-                    if (item->email && item->email->subject)
-                        printf("\tSUMMARY: %s", pst_rfc2426_escape(item->email->subject->subj));
+                    if (item->subject.str)
+                        printf("\tSUMMARY: %s", pst_rfc2426_escape(item->subject.str, &result, &resultlen));
                     if (item->appointment->start)
-                        printf("\tSTART: %s", pst_rfc2445_datetime_format(item->appointment->start));
+                        printf("\tSTART: %s", pst_rfc2445_datetime_format(item->appointment->start, sizeof(time_buffer), time_buffer));
                     if (item->appointment->end)
-                        printf("\tEND: %s", pst_rfc2445_datetime_format(item->appointment->end));
+                        printf("\tEND: %s", pst_rfc2445_datetime_format(item->appointment->end, sizeof(time_buffer), time_buffer));
                     printf("\tALL DAY: %s", (item->appointment->all_day==1 ? "Yes" : "No"));
                     printf("\n");
 
                 } else {
                     ff.skip_count++;
-                    DEBUG_MAIN(("main: Unknown item type. %i. Ascii1=\"%s\"\n",
+                    DEBUG_INFO(("Unknown item type. %i. Ascii1=\"%s\"\n",
                                       item->type, item->ascii_type));
                 }
                 pst_freeItem(item);
             } else {
                 ff.skip_count++;
-                DEBUG_MAIN(("main: A NULL item was seen\n"));
+                DEBUG_INFO(("A NULL item was seen\n"));
             }
             d_ptr = d_ptr->next;
         }
     }
     close_enter_dir(&ff);
+    if (result) free(result);
     DEBUG_RET();
 }
 
@@ -169,7 +177,7 @@ void version() {
 
 int main(int argc, char* const* argv) {
     pst_item *item = NULL;
-    pst_desc_ll *d_ptr;
+    pst_desc_tree *d_ptr;
     char *temp  = NULL; //temporary char pointer
     int  c;
     char *d_log = NULL;
@@ -198,8 +206,7 @@ int main(int argc, char* const* argv) {
         // force a log file
         if (!d_log) d_log = "lspst.log";
     #endif // defined DEBUG_ALL
-    DEBUG_INIT(d_log);
-    DEBUG_REGISTER_CLOSE();
+    DEBUG_INIT(d_log, NULL);
     DEBUG_ENT("main");
 
 	if (argc <= optind) {
@@ -216,14 +223,14 @@ int main(int argc, char* const* argv) {
     pst_load_extended_attributes(&pstfile);
 
     d_ptr = pstfile.d_head; // first record is main record
-    item  = pst_parse_item(&pstfile, d_ptr);
+    item  = pst_parse_item(&pstfile, d_ptr, NULL);
     if (!item || !item->message_store) {
         DEBUG_RET();
-        DIE(("main: Could not get root record\n"));
+        DIE(("Could not get root record\n"));
     }
 
     // default the file_as to the same as the main filename if it doesn't exist
-    if (!item->file_as) {
+    if (!item->file_as.str) {
         if (!(temp = strrchr(argv[1], '/')))
             if (!(temp = strrchr(argv[1], '\\')))
                 temp = argv[1];
@@ -231,14 +238,12 @@ int main(int argc, char* const* argv) {
                 temp++; // get past the "\\"
         else
             temp++; // get past the "/"
-        item->file_as = (char*)xmalloc(strlen(temp)+1);
-        strcpy(item->file_as, temp);
+        item->file_as.str = strdup(temp);
+        item->file_as.is_utf8 = 1;
     }
-    fprintf(stderr, "item->file_as = '%s'.\n", item->file_as);
 
     d_ptr = pst_getTopOfFolders(&pstfile, item);
     if (!d_ptr) DIE(("Top of folders record not found. Cannot continue\n"));
-    DEBUG_MAIN(("d_ptr(TOF) = %p.\n", d_ptr));
 
     process(item, d_ptr->child);    // do the childred of TOPF
     pst_freeItem(item);
@@ -263,48 +268,3 @@ void canonicalize_filename(char *fname) {
 }
 
 
-void debug_print(char *fmt, ...) {
-    // shamlessly stolen from minprintf() in K&R pg. 156
-    va_list ap;
-    char *p, *sval;
-    void *pval;
-    int ival;
-    double dval;
-    FILE *fp = stderr;
-
-    va_start(ap, fmt);
-    for(p = fmt; *p; p++) {
-        if (*p != '%') {
-            fputc(*p, fp);
-            continue;
-        }
-        switch (tolower(*++p)) {
-            case 'd': case 'i':
-                ival = va_arg(ap, int);
-                fprintf(fp, "%d", ival);
-                break;
-            case 'f':
-                dval = va_arg(ap, double);
-                fprintf(fp, "%f", dval);
-                break;
-            case 's':
-                for (sval = va_arg(ap, char *); *sval; ++sval)
-                    fputc(*sval, fp);
-                break;
-            case 'p':
-                pval = va_arg(ap, void *);
-                fprintf(fp, "%p", pval);
-                break;
-            case 'x':
-                ival = va_arg(ap, int);
-                fprintf(fp, "%#010x", ival);
-                break;
-            default:
-                fputc(*p, fp);
-                break;
-        }
-    }
-    va_end(ap);
-}
-
-
