@@ -98,7 +98,8 @@ static void usage();
 static void usage()
 {
     version();
-    printf("Usage: %s [OPTIONS] {PST FILENAME}\n", prog_name);
+    printf("Usage: %s -f ttf-font-file [OPTIONS] {PST FILENAME}\n", prog_name);
+    printf("\t-f ttf-font-file  \t- Set the font file\n");
     printf("OPTIONS:\n");
     printf("\t-B bates-prefix   \t- Set the bates prefix string\n");
     printf("\t-O dii-output-file\t- Set the dii load file output filename\n");
@@ -106,7 +107,6 @@ static void usage()
     printf("\t-b bates-number   \t- Set the starting bates sequence number\n");
     printf("\t-c bates-color    \t- Specify the color of the bates stamps as 6 digit hex\n");
     printf("\t-d filename       \t- Debug to file. This is a binary log. Use readpstlog to print it.\n");
-    printf("\t-f ttf-font-file  \t- Set the font file\n");
     printf("\t-h                \t- Help. This screen\n");
     printf("\t-o dirname        \t- Output directory to write files to.\n");
 }
@@ -175,20 +175,20 @@ static string write_separate_attachment(string fname, pst_item_attach* current_a
 
     // If there is a long filename (filename2) use that, otherwise
     // use the 8.3 filename (filename1)
-    char *attach_filename = (current_attach->filename2) ? current_attach->filename2
-                                                        : current_attach->filename1;
+    char *attach_filename = (current_attach->filename2.str) ? current_attach->filename2.str
+                                                            : current_attach->filename1.str;
     DEBUG_ENT("write_separate_attachment");
     check_filename(fname);
     const char* f_name = fname.c_str();
-    DEBUG_EMAIL(("dirname=%s, pathname=%s, filename=%s\n", output_directory, f_name, attach_filename));
+    DEBUG_INFO(("dirname=%s, pathname=%s, filename=%s\n", output_directory, f_name, attach_filename));
     int len = strlen(output_directory) + 1 + strlen(f_name) + 15;
     if (!attach_filename) {
         // generate our own (dummy) filename for the attachement
-        temp = (char*)xmalloc(len);
+        temp = (char*)pst_malloc(len);
         sprintf(temp, "%s/%s_attach%i", output_directory, f_name, attach_num);
     } else {
         // have an attachment name, make sure it's unique
-        temp = (char*)xmalloc(len+strlen(attach_filename));
+        temp = (char*)pst_malloc(len+strlen(attach_filename));
         do {
             if (fp) fclose(fp);
             if (x == 0)
@@ -200,15 +200,11 @@ static string write_separate_attachment(string fname, pst_item_attach* current_a
             DIE(("error finding attachment name. exhausted possibilities to %s\n", temp));
         }
     }
-    DEBUG_EMAIL(("Saving attachment to %s\n", temp));
+    DEBUG_INFO(("Saving attachment to %s\n", temp));
     if (!(fp = fopen(temp, "wb"))) {
-        WARN(("write_separate_attachment: Cannot open attachment save file \"%s\"\n", temp));
+        DEBUG_WARN(("write_separate_attachment: Cannot open attachment save file \"%s\"\n", temp));
     } else {
-        if (current_attach->data)
-            pst_fwrite(current_attach->data, 1, current_attach->size, fp);
-        else {
-            (void)pst_attach_to_file(pst, current_attach, fp);
-        }
+        (void)pst_attach_to_file(pst, current_attach, fp);
         fclose(fp);
     }
     string rc(temp);
@@ -240,15 +236,15 @@ static void print_pdf_short(const char *line, int len, int color)
     while ((p = strchr(l, '&'))) {
         *p = '\0';
         char *err = gdImageStringFTEx(image, &brect[0], color, font_file, sz, 0.0, x_position, y_position, l, &strex);
-        if (err) printf(err);
+        if (err) printf("%s", err);
         x_position += (brect[2]-brect[6]);
         l = p+1;
         err = gdImageStringFTEx(image, &brect[0], color, font_file, sz, 0.0, x_position, y_position, (char*)"&amp;", &strex);
-        if (err) printf(err);
+        if (err) printf("%s", err);
         x_position += (brect[2]-brect[6]);
     }
     char *err = gdImageStringFTEx(image, &brect[0], color, font_file, sz, 0.0, x_position, y_position, l, &strex);
-    if (err) printf(err);
+    if (err) printf("%s", err);
     x_position += (brect[2]-brect[6]);
     col_number += len;
 }
@@ -331,7 +327,7 @@ static void open_png()
 
         char line[LINE_SIZE];
         char *err = gdImageStringFTEx(NULL, &brect[0], black, font_file, sz, 0.0, margin, margin, (char*)"LMgqQ", &strex);
-        if (err) printf(err);
+        if (err) printf("%s", err);
         line_height = (brect[3]-brect[7]) * 12/10;
         char_width  = (brect[2]-brect[6]) / 5;
         col_number  = 0;
@@ -384,7 +380,7 @@ static void close_pdf()
 {
     close_png();
     conversion += string(" ") + pdf_name;
-    system(conversion.c_str());
+    (void)system(conversion.c_str());
     for (vector<string>::iterator i=png_names.begin(); i!=png_names.end(); i++) {
         remove((*i).c_str());
     }
@@ -445,17 +441,17 @@ static void write_normal_email(file_ll &f, pst_item* item, pst_file* pst)
 {
     DEBUG_ENT("write_normal_email");
     char *soh = NULL;  // real start of headers.
-    if (item->email->header) {
+    if (item->email->header.str) {
         // some of the headers we get from the file are not properly defined.
         // they can contain some email stuff too. We will cut off the header
         // when we see a \n\n or \r\n\r\n
-        removeCR(item->email->header);
-        char *temp = strstr(item->email->header, "\n\n");
+        removeCR(item->email->header.str);
+        char *temp = strstr(item->email->header.str, "\n\n");
         if (temp) {
-            DEBUG_EMAIL(("Found body text in header\n"));
+            DEBUG_INFO(("Found body text in header\n"));
             temp[1] = '\0'; // stop after first \n
         }
-        soh = skip_header_prologue(item->email->header);
+        soh = skip_header_prologue(item->email->header.str);
     }
 
     char folder_line[LINE_SIZE];
@@ -471,16 +467,16 @@ static void write_normal_email(file_ll &f, pst_item* item, pst_file* pst)
     string myto   = extract_header(soh, "To");
     string mycc   = extract_header(soh, "Cc");
     string mybcc  = extract_header(soh, "Bcc");
-    if (myfrom.empty()) write_simple("FROM", item->email->outlook_sender_name, item->email->sender_address);
+    if (myfrom.empty()) write_simple("FROM", item->email->outlook_sender_name.str, item->email->sender_address.str);
     else                write_simple("FROM", myfrom);
-    if (myto.empty())   write_simple("TO",   item->email->sentto_address,      item->email->recip_address);
+    if (myto.empty())   write_simple("TO",   item->email->sentto_address.str,      item->email->recip_address.str);
     else                write_simple("TO", myto);
-    if (mycc.empty())   write_simple("CC",   item->email->cc_address);
+    if (mycc.empty())   write_simple("CC",   item->email->cc_address.str);
     else                write_simple("CC", mycc);
-    if (mybcc.empty())  write_simple("BCC",  item->email->bcc_address);
+    if (mybcc.empty())  write_simple("BCC",  item->email->bcc_address.str);
     else                write_simple("BCC", mybcc);
     if (item->email->sent_date) {
-        time_t t = fileTimeToUnixTime(item->email->sent_date, NULL);
+        time_t t = pst_fileTimeToUnixTime(item->email->sent_date);
         char c_time[C_TIME_SIZE];
         strftime(c_time, C_TIME_SIZE, "%F", gmtime(&t));
         write_simple("DATESENT", c_time);
@@ -488,26 +484,24 @@ static void write_normal_email(file_ll &f, pst_item* item, pst_file* pst)
         write_simple("TIMESENT", c_time);
     }
     if (item->email->arrival_date) {
-        time_t t = fileTimeToUnixTime(item->email->arrival_date, NULL);
+        time_t t = pst_fileTimeToUnixTime(item->email->arrival_date);
         char c_time[C_TIME_SIZE];
         strftime(c_time, C_TIME_SIZE, "%F", gmtime(&t));
         write_simple("DATERCVD", c_time);
         strftime(c_time, C_TIME_SIZE, "%T+0000", gmtime(&t));
         write_simple("TIMERCVD", c_time);
     }
-    if (item->email->subject) {
-        write_simple("SUBJECT", item->email->subject->subj);
-    }
-    write_simple("MSGID", item->email->messageid);
-    if (item->email->flag) {
-        write_simple("READ", (item->email->flag & 1) ? "Y" : "N");
+    if (item->subject.str) {
+        write_simple("SUBJECT", item->subject.str);
     }
+    write_simple("MSGID", item->email->messageid.str);
+    write_simple("READ", (item->flags & 1) ? "Y" : "N");
 
-    DEBUG_EMAIL(("About to print Header\n"));
+    DEBUG_INFO(("About to print Header\n"));
     fprintf(dii_file, "@HEADER\n");
 
-    if (item && item->email && item->email->subject && item->email->subject->subj) {
-        DEBUG_EMAIL(("item->email->subject->subj = %s\n", item->email->subject->subj));
+    if (item && item->subject.str) {
+        DEBUG_INFO(("item->subject = %s\n", item->subject.str));
     }
 
     if (soh) {
@@ -521,28 +515,28 @@ static void write_normal_email(file_ll &f, pst_item* item, pst_file* pst)
 
     } else {
         //make up our own headers
-        const char *temp = item->email->outlook_sender;
+        const char *temp = item->email->outlook_sender.str;
         if (!temp) temp = "";
-        snprintf(line, sizeof(line), "From: \"%s\" <%s>\n", item->email->outlook_sender_name, temp);
+        snprintf(line, sizeof(line), "From: \"%s\" <%s>\n", item->email->outlook_sender_name.str, temp);
         print_pdf(line);
 
-        if (item->email->subject) {
-            snprintf(line, sizeof(line), "Subject: %s\n", item->email->subject->subj);
+        if (item->subject.str) {
+            snprintf(line, sizeof(line), "Subject: %s\n", item->subject.str);
         } else {
             snprintf(line, sizeof(line), "Subject: \n");
         }
         print_pdf(line);
 
-        snprintf(line, sizeof(line), "To: %s\n", item->email->sentto_address);
+        snprintf(line, sizeof(line), "To: %s\n", item->email->sentto_address.str);
         print_pdf(line);
 
-        if (item->email->cc_address) {
-            snprintf(line, sizeof(line), "Cc: %s\n", item->email->cc_address);
+        if (item->email->cc_address.str) {
+            snprintf(line, sizeof(line), "Cc: %s\n", item->email->cc_address.str);
             print_pdf(line);
         }
 
         if (item->email->sent_date) {
-            time_t em_time = fileTimeToUnixTime(item->email->sent_date, 0);
+            time_t em_time = pst_fileTimeToUnixTime(item->email->sent_date);
             char c_time[C_TIME_SIZE];
             strftime(c_time, C_TIME_SIZE, "%a, %d %b %Y %H:%M:%S %z", gmtime(&em_time));
             snprintf(line, sizeof(line), "Date: %s\n", c_time);
@@ -553,27 +547,31 @@ static void write_normal_email(file_ll &f, pst_item* item, pst_file* pst)
     print_pdf_only(line, black);
     fprintf(dii_file, "@HEADER-END\n");
 
-    DEBUG_EMAIL(("About to print Body\n"));
+    DEBUG_INFO(("About to print Body\n"));
     fprintf(dii_file, "@EMAIL-BODY\n");
-    if (item->email->body) {
-        removeCR(item->email->body);
-        print_pdf(item->email->body);
-    } else if (item->email->htmlbody) {
-        removeCR(item->email->htmlbody);
-        print_pdf(item->email->htmlbody);
-    } else if (item->email->encrypted_body || item->email->encrypted_htmlbody) {
-        print_pdf("The body of this email is encrypted. This isn't supported yet, but the body is now an attachment\n");
+    if (item->body.str) {
+        removeCR(item->body.str);
+        print_pdf(item->body.str);
+    } else if (item->email->htmlbody.str) {
+        removeCR(item->email->htmlbody.str);
+        print_pdf(item->email->htmlbody.str);
+    } else if (item->email->encrypted_body.data || item->email->encrypted_htmlbody.data) {
+        char ln[LINE_SIZE];
+        snprintf(ln, sizeof(ln), "%s", "The body of this email is encrypted. This isn't supported yet, but the body is now an attachment\n");
+        print_pdf(ln);
     }
     fprintf(dii_file, "@EMAIL-END\n");
 
     int attach_num = 0;
-    for (pst_item_attach* current_attach = item->attach; current_attach; current_attach = current_attach->next) {
-        DEBUG_EMAIL(("Attempting Attachment encoding\n"));
-        if (!current_attach->data) {
-            DEBUG_EMAIL(("Data of attachment is NULL!. Size is supposed to be %i\n", current_attach->size));
+    for (pst_item_attach* attach = item->attach; attach; attach = attach->next) {
+        pst_convert_utf8_null(item, &attach->filename1);
+        pst_convert_utf8_null(item, &attach->filename2);
+        pst_convert_utf8_null(item, &attach->mimetype);
+        DEBUG_INFO(("Attempting Attachment encoding\n"));
+        if (attach->data.data || attach->i_id) {
+            string an = write_separate_attachment(f.name, attach, ++attach_num, pst);
+            fprintf(dii_file, "@EATTACH %s\n", an.c_str());
         }
-        string an = write_separate_attachment(f.name, current_attach, ++attach_num, pst);
-        fprintf(dii_file, "@EATTACH %s\n", an.c_str());
     }
     close_pdf();
     fprintf(dii_file, "@BATESBEG %d\n", bates);
@@ -585,11 +583,10 @@ static void write_normal_email(file_ll &f, pst_item* item, pst_file* pst)
 static void create_enter_dir(file_ll &f, file_ll *parent, pst_item *item);
 static void create_enter_dir(file_ll &f, file_ll *parent, pst_item *item)
 {
-    f.email_count  = 0;
-    f.skip_count   = 0;
+    pst_convert_utf8(item, &item->file_as);
     f.type         = item->type;
-    f.stored_count = (item->folder) ? item->folder->email_count : 0;
-    f.name         = ((parent) ? parent->name + "/" : "") + string(item->file_as);
+    f.stored_count = (item->folder) ? item->folder->item_count : 0;
+    f.name         = ((parent) ? parent->name + "/" : "") + string(item->file_as.str);
 }
 
 
@@ -599,37 +596,35 @@ static void close_enter_dir(file_ll &f)
 }
 
 
-static void process(pst_item *outeritem, file_ll *parent, pst_desc_ll *d_ptr);
-static void process(pst_item *outeritem, file_ll *parent, pst_desc_ll *d_ptr)
+static void process(pst_item *outeritem, file_ll *parent, pst_desc_tree *d_ptr);
+static void process(pst_item *outeritem, file_ll *parent, pst_desc_tree *d_ptr)
 {
     file_ll ff;
     pst_item *item = NULL;
     DEBUG_ENT("process");
     create_enter_dir(ff, parent, outeritem);
-    while (d_ptr) {
+    for (; d_ptr; d_ptr = d_ptr->next) {
         if (d_ptr->desc) {
-            item = pst_parse_item(&pstfile, d_ptr);
+            item = pst_parse_item(&pstfile, d_ptr, NULL);
             DEBUG_INFO(("item pointer is %p\n", item));
             if (item) {
-                if (item->folder && d_ptr->child )  {
+                if (item->folder && item->file_as.str && d_ptr->child )  {
                     //if this is a non-empty folder, we want to recurse into it
-                    fprintf(stderr, "entering folder %s\n", item->file_as);
+                    fprintf(stderr, "entering folder %s\n", item->file_as.str);
                     process(item, &ff, d_ptr->child);
-                } else if (item->email && (item->type == PST_TYPE_NOTE || item->type == PST_TYPE_REPORT || item->type == PST_TYPE_OTHER)) {
+                } else if (item->email && (item->type == PST_TYPE_NOTE || item->type == PST_TYPE_SCHEDULE || item->type == PST_TYPE_REPORT)) {
                     ff.email_count++;
-                    DEBUG_MAIN(("main: Processing Email\n"));
-                    if ((ff.type != PST_TYPE_NOTE) && (ff.type != PST_TYPE_REPORT) && (ff.type != PST_TYPE_OTHER)) {
-                        DEBUG_MAIN(("main: I have an email, but the folder isn't an email folder. Processing anyway\n"));
-                    }
                     write_normal_email(ff, item, &pstfile);
                 }
+                else {
+                    ff.skip_count++;    // other mapi objects
+                }
                 pst_freeItem(item);
             } else {
                 ff.skip_count++;
-                DEBUG_MAIN(("main: A NULL item was seen\n"));
+                DEBUG_INFO(("A NULL item was seen\n"));
             }
         }
-        d_ptr = d_ptr->next;
     }
     close_enter_dir(ff);
     DEBUG_RET();
@@ -638,7 +633,7 @@ static void process(pst_item *outeritem, file_ll *parent, pst_desc_ll *d_ptr)
 
 int main(int argc, char* const* argv)
 {
-    pst_desc_ll *d_ptr;
+    pst_desc_tree *d_ptr;
     char *fname = NULL;
     char c;
     char *d_log = NULL;
@@ -683,6 +678,11 @@ int main(int argc, char* const* argv)
         }
     }
 
+    if (!font_file) {
+        usage();
+        exit(1);
+    }
+
     if (argc > optind) {
         fname = argv[optind];
     } else {
@@ -695,8 +695,7 @@ int main(int argc, char* const* argv)
         // force a log file
         if (!d_log) d_log = "pst2dii.log";
     #endif
-    DEBUG_INIT(d_log);
-    DEBUG_REGISTER_CLOSE();
+    DEBUG_INIT(d_log, NULL);
     DEBUG_ENT("main");
     RET_DERROR(pst_open(&pstfile, fname), 1, ("Error opening File\n"));
     RET_DERROR(pst_load_index(&pstfile), 2, ("Index Error\n"));
@@ -704,10 +703,10 @@ int main(int argc, char* const* argv)
     pst_load_extended_attributes(&pstfile);
 
     d_ptr = pstfile.d_head; // first record is main record
-    item  = (pst_item*)pst_parse_item(&pstfile, d_ptr);
+    item  = (pst_item*)pst_parse_item(&pstfile, d_ptr, NULL);
     if (!item || !item->message_store) {
         DEBUG_RET();
-        DIE(("main: Could not get root record\n"));
+        DIE(("Could not get root record\n"));
     }
 
     d_ptr = pst_getTopOfFolders(&pstfile, item);
